@@ -32,11 +32,7 @@ from swift.common.utils import config_true_value, LogAdapter
 import swiftclient as client
 from swift.common import direct_client
 from swift.common.http import HTTP_CONFLICT
-
-try:
-    import simplejson as json
-except ImportError:
-    import json
+from swift.common.utils import json
 
 
 def _func_on_containers(logger, conf, concurrency_key, func):
@@ -70,6 +66,40 @@ def create_containers(logger, conf):
     _func_on_containers(logger, conf, 'put_concurrency', client.put_container)
 
 
+class SourceFile(object):
+    """
+    Iterable, file-like object to lazily emit a bunch of zeros in
+    reasonable-size chunks.
+
+    swift.common.direct_client wants iterables, but swiftclient wants
+    file-like objects where hasattr(thing, 'read') is true. Therefore,
+    this class can do both.
+    """
+
+    def __init__(self, size, chunk_size=1024 * 64):
+        self.pos = 0
+        self.size = size
+        self.chunk_size = chunk_size
+
+    def __iter__(self):
+        return self
+
+    def __len__(self):
+        return self.size
+
+    def next(self):
+        if self.pos >= self.size:
+            raise StopIteration
+        chunk_size = min(self.size - self.pos, self.chunk_size)
+        yield '0' * chunk_size
+        self.pos += chunk_size
+
+    def read(self, desired_size):
+        chunk_size = min(self.size - self.pos, desired_size)
+        self.pos += chunk_size
+        return '0' * chunk_size
+
+
 class ConnectionPool(eventlet.pools.Pool):
 
     def __init__(self, url, size):
@@ -259,9 +289,13 @@ class DistributedBenchController(object):
             'INFO (\d+) (.*) \*\*FINAL\*\* \[(\d+) failures\], (\d+\.\d+)/s')
         self.clients = conf.bench_clients
         del conf.bench_clients
-        for k in ['put_concurrency', 'get_concurrency', 'del_concurrency',
-                  'num_objects', 'num_gets']:
-            setattr(conf, k, max(1, int(getattr(conf, k)) / len(self.clients)))
+        for key, minval in [('put_concurrency', 1),
+                            ('get_concurrency', 1),
+                            ('del_concurrency', 1),
+                            ('num_objects', 0),
+                            ('num_gets', 0)]:
+            setattr(conf, key,
+                    max(minval, int(getattr(conf, key)) / len(self.clients)))
         self.conf = conf
 
     def run(self):
@@ -423,10 +457,10 @@ class BenchPUT(Bench):
         if self.object_sources:
             source = random.choice(self.files)
         elif self.upper_object_size > self.lower_object_size:
-            source = '0' * random.randint(self.lower_object_size,
-                                          self.upper_object_size)
+            source = SourceFile(random.randint(self.lower_object_size,
+                                               self.upper_object_size))
         else:
-            source = '0' * self.object_size
+            source = SourceFile(self.object_size)
         device = random.choice(self.devices)
         partition = str(random.randint(1, 3000))
         container_name = random.choice(self.containers)
