@@ -186,6 +186,8 @@ class InternalClient(object):
         """
 
         resp = self.make_request('HEAD', path, {}, acceptable_statuses)
+        if not resp.status_int // 100 == 2:
+            return {}
         metadata_prefix = metadata_prefix.lower()
         metadata = {}
         for k, v in resp.headers.iteritems():
@@ -212,20 +214,23 @@ class InternalClient(object):
         :raises Exception: Exception is raised when code fails in an
                            unexpected way.
         """
-
+        if isinstance(marker, unicode):
+            marker = marker.encode('utf8')
+        if isinstance(end_marker, unicode):
+            end_marker = end_marker.encode('utf8')
         while True:
             resp = self.make_request(
                 'GET', '%s?format=json&marker=%s&end_marker=%s' %
                 (path, quote(marker), quote(end_marker)),
                 {}, acceptable_statuses)
-            if resp.status_int != 200:
+            if not resp.status_int == 200:
                 break
             data = json.loads(resp.body)
             if not data:
                 break
             for item in data:
                 yield item
-            marker = data[-1]['name']
+            marker = data[-1]['name'].encode('utf8')
 
     def make_path(self, account, container=None, obj=None):
         """
@@ -329,6 +334,8 @@ class InternalClient(object):
 
         path = self.make_path(account)
         resp = self.make_request('HEAD', path, {}, acceptable_statuses)
+        if not resp.status_int // 100 == 2:
+            return (0, 0)
         return (int(resp.headers.get('x-account-container-count', 0)),
                 int(resp.headers.get('x-account-object-count', 0)))
 
@@ -401,7 +408,7 @@ class InternalClient(object):
 
         path = self.make_path(account, container)
         resp = self.make_request('HEAD', path, {}, (2, HTTP_NOT_FOUND))
-        return resp.status_int != HTTP_NOT_FOUND
+        return not resp.status_int == HTTP_NOT_FOUND
 
     def create_container(
             self, account, container, headers=None, acceptable_statuses=(2,)):
@@ -591,8 +598,9 @@ class InternalClient(object):
 
         headers = headers or {}
         path = self.make_path(account, container, obj)
-
         resp = self.make_request('GET', path, headers, acceptable_statuses)
+        if not resp.status_int // 100 == 2:
+            return
 
         last_part = ''
         compressed = obj.endswith('.gz')
