@@ -1,4 +1,4 @@
-# Copyright (c) 2012 OpenStack, LLC.
+# Copyright (c) 2013 OpenStack, LLC.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,11 +15,13 @@
 
 import tarfile
 from urllib import quote, unquote
-from swift.common.swob import Request, HTTPException, HTTPBadGateway, \
+from xml.sax import saxutils
+from swift.common.swob import Request, HTTPBadGateway, \
     HTTPCreated, HTTPBadRequest, HTTPNotFound, HTTPUnauthorized, HTTPOk, \
-    HTTPPreconditionFailed, HTTPRequestEntityTooLarge
-from swift.common.utils import split_path, json, TRUE_VALUES
-from swift.common.constraints import check_utf8
+    HTTPPreconditionFailed, HTTPRequestEntityTooLarge, HTTPNotAcceptable, \
+    HTTPLengthRequired, wsgify
+from swift.common.utils import json, TRUE_VALUES
+from swift.common.constraints import check_utf8, MAX_FILE_SIZE
 from swift.common.http import HTTP_BAD_REQUEST, HTTP_UNAUTHORIZED, \
     HTTP_NOT_FOUND
 from swift.common.constraints import MAX_OBJECT_NAME_LENGTH, \
@@ -36,6 +38,45 @@ class CreateContainerError(Exception):
         Exception.__init__(self, msg)
 
 
+ACCEPTABLE_FORMATS = ['text/plain', 'application/json', 'application/xml',
+                      'text/xml']
+
+
+def get_response_body(data_format, data_dict, error_list):
+    """
+    Returns a properly formatted response body according to format.
+    :params data_format: resulting format
+    :params data_dict: generated data about results.
+    :params error_list: list of quoted filenames that failed
+    """
+    if data_format == 'text/plain':
+        output = ''
+        for key in sorted(data_dict.keys()):
+            output += '%s: %s\n' % (key, data_dict[key])
+        output += 'Errors:\n'
+        output += '\n'.join(
+            ['%s, %s' % (name, status)
+             for name, status in error_list])
+        return output
+    if data_format == 'application/json':
+        data_dict['Errors'] = error_list
+        return json.dumps(data_dict)
+    if data_format.endswith('/xml'):
+        output = '<?xml version="1.0" encoding="UTF-8"?>\n<delete>\n'
+        for key in sorted(data_dict.keys()):
+            xml_key = key.replace(' ', '_').lower()
+            output += '<%s>%s</%s>\n' % (xml_key, data_dict[key], xml_key)
+        output += '<errors>\n'
+        output += '\n'.join(
+            ['<object>'
+             '<name>%s</name><status>%s</status>'
+             '</object>' % (saxutils.escape(name), status) for
+             name, status in error_list])
+        output += '</errors>\n</delete>\n'
+        return output
+    raise HTTPNotAcceptable('Invalid output type')
+
+
 class Bulk(object):
     """
     Middleware that will do many operations on a single request.
@@ -43,12 +84,12 @@ class Bulk(object):
     Extract Archive:
 
     Expand tar files into a swift account. Request must be a PUT with the
-    header X-Extract-Archive specifying the format of archive file. Accepted
-    formats are tar, tar.gz, and tar.bz2.
+    query parameter ?extract-archive=format specifying the format of archive
+    file. Accepted formats are tar, tar.gz, and tar.bz2.
 
     For a PUT to the following url:
 
-    /v1/AUTH_Account/$UPLOAD_PATH
+    /v1/AUTH_Account/$UPLOAD_PATH?extract-archive=tar.gz
 
     UPLOAD_PATH is where the files will be expanded to. UPLOAD_PATH can be a
     container, a pseudo-directory within a container, or an empty string. The
@@ -58,7 +99,7 @@ class Bulk(object):
 
     Where FILE_PATH is the file name from the listing in the tar file.
 
-    If the UPLOAD_PATH is empty string, containers will be auto created
+    If the UPLOAD_PATH is an empty string, containers will be auto created
     accordingly and files in the tar that would not map to any container (files
     in the base directory) will be ignored.
 
@@ -67,25 +108,49 @@ class Bulk(object):
 
     If all valid files were uploaded successfully will return an HTTPCreated
     response. If any files failed to be created will return an HTTPBadGateway
-    response. In both cases the response body is a json dictionary specifying
-    in the number of files successfully uploaded and a list of the files that
-    failed.
+    response. In both cases the response body will specify the number of files
+    successfully uploaded and a list of the files that failed. The return body
+    will be formatted in the way specified in the request's Accept header.
+    Acceptable formats are text/plain, application/json, application/xml, and
+    text/xml.
+
+    There are proxy logs created for each file (which becomes a subrequest) in
+    the tar. The subrequest's proxy log will have a swift.source set to "EA"
+    the log's content length will reflect the unzipped size of the file. If
+    double proxy-logging is used the leftmost logger will not have a
+    swift.source set and the content length will reflect the size of the
+    payload sent to the proxy (the unexpanded size of the tar.gz).
 
     Bulk Delete:
 
-    Will delete multiple objects from their account with a single request.
-    Responds to DELETE requests with a header 'X-Bulk-Delete: true'.
+    Will delete multiple objects or containers from their account with a
+    single request. Responds to DELETE requests with query parameter
+    ?bulk-delete set. The Content-Type should be set to text/plain.
     The body of the DELETE request will be a newline separated list of url
     encoded objects to delete. You can only delete 1000 (configurable) objects
-    per file. The objects listed must be URL encoded and in the form:
+    per request. The objects specified in the DELETE request body must be URL
+    encoded and in the form:
 
     /container_name/obj_name
 
-    If all objects were successfully deleted (or did not exist), will return an
-    HTTPOk.  If any objects failed to delete, will return an HTTPBadGateway. In
-    both cases the response body is a json dictionary specifying in the number
-    of objects successfully deleted, not found, and a list of the objects that
-    failed.
+    or for a container (which must be empty at time of delete)
+
+    /container_name
+
+    If all items were successfully deleted (or did not exist), will return an
+    HTTPOk. If any failed to delete, will return an HTTPBadGateway. In
+    both cases the response body will specify the number of items
+    successfully deleted, not found, and a list of those that failed.
+    The return body will be formatted in the way specified in the request's
+    Accept header. Acceptable formats are text/plain, application/json,
+    application/xml, and text/xml.
+
+    There are proxy logs created for each object or container (which becomes a
+    subrequest) that is deleted. The subrequest's proxy log will have a
+    swift.source set to "BD" the log's content length of 0. If double
+    proxy-logging is used the leftmost logger will not have a
+    swift.source set and the content length will reflect the size of the
+    payload sent to the proxy (the list of objects/containers to be deleted).
     """
 
     def __init__(self, app, conf):
@@ -106,6 +171,7 @@ class Bulk(object):
         """
         new_env = req.environ.copy()
         new_env['PATH_INFO'] = container_path
+        new_env['swift.source'] = 'EA'
         create_cont_req = Request.blank(container_path, environ=new_env)
         resp = create_cont_req.get_response(self.app)
         if resp.status_int // 100 != 2:
@@ -123,14 +189,14 @@ class Bulk(object):
         line = ''
         data_remaining = True
         objs_to_delete = []
+        if req.content_length is None and \
+                req.headers.get('transfer-encoding', '').lower() != 'chunked':
+            raise HTTPLengthRequired(request=req)
+
         while data_remaining:
-            if len(objs_to_delete) > self.max_deletes_per_request:
-                raise HTTPRequestEntityTooLarge(
-                    'Maximum Bulk Deletes: %d per request' %
-                    self.max_deletes_per_request)
             if '\n' in line:
                 obj_to_delete, line = line.split('\n', 1)
-                objs_to_delete.append(obj_to_delete)
+                objs_to_delete.append(unquote(obj_to_delete))
             else:
                 data = req.body_file.read(MAX_PATH_LENGTH)
                 if data:
@@ -138,23 +204,37 @@ class Bulk(object):
                 else:
                     data_remaining = False
                     if line.strip():
-                        objs_to_delete.append(line)
+                        objs_to_delete.append(unquote(line))
+            if len(objs_to_delete) > self.max_deletes_per_request:
+                raise HTTPRequestEntityTooLarge(
+                    'Maximum Bulk Deletes: %d per request' %
+                    self.max_deletes_per_request)
             if len(line) > MAX_PATH_LENGTH * 2:
                 raise HTTPBadRequest('Invalid File Name')
         return objs_to_delete
 
-    def handle_delete(self, req):
+    def handle_delete(self, req, objs_to_delete=None, user_agent='BulkDelete',
+                      swift_source='BD'):
         """
         :params req: a swob Request
         :raises HTTPException: on unhandled errors
         :returns: a swob Response
         """
         try:
-            vrs, account, _junk = split_path(unquote(req.path), 2, 3, True)
+            vrs, account, _junk = req.split_path(2, 3, True)
         except ValueError:
             return HTTPNotFound(request=req)
 
-        objs_to_delete = self.get_objs_to_delete(req)
+        incoming_format = req.headers.get('Content-Type')
+        if incoming_format and not incoming_format.startswith('text/plain'):
+            # For now only accept newline separated object names
+            return HTTPNotAcceptable(request=req)
+        out_content_type = req.accept.best_match(ACCEPTABLE_FORMATS)
+        if not out_content_type:
+            return HTTPNotAcceptable(request=req)
+
+        if objs_to_delete is None:
+            objs_to_delete = self.get_objs_to_delete(req)
         failed_files = []
         success_count = not_found_count = 0
         failed_file_response_type = HTTPBadRequest
@@ -162,7 +242,6 @@ class Bulk(object):
             obj_to_delete = obj_to_delete.strip().lstrip('/')
             if not obj_to_delete:
                 continue
-            obj_to_delete = unquote(obj_to_delete)
             delete_path = '/'.join(['', vrs, account, obj_to_delete])
             if not check_utf8(delete_path):
                 failed_files.append([quote(delete_path),
@@ -172,6 +251,9 @@ class Bulk(object):
             new_env['PATH_INFO'] = delete_path
             del(new_env['wsgi.input'])
             new_env['CONTENT_LENGTH'] = 0
+            new_env['HTTP_USER_AGENT'] = \
+                '%s %s' % (req.environ.get('HTTP_USER_AGENT'), user_agent)
+            new_env['swift.source'] = swift_source
             delete_obj_req = Request.blank(delete_path, new_env)
             resp = delete_obj_req.get_response(self.app)
             if resp.status_int // 100 == 2:
@@ -183,17 +265,18 @@ class Bulk(object):
             else:
                 if resp.status_int // 100 == 5:
                     failed_file_response_type = HTTPBadGateway
-                failed_files.append([delete_path, resp.status])
+                failed_files.append([quote(delete_path), resp.status])
 
-        resp_body = json.dumps(
+        resp_body = get_response_body(
+            out_content_type,
             {'Number Deleted': success_count,
-             'Number Not Found': not_found_count,
-             'Errors': failed_files})
+             'Number Not Found': not_found_count},
+            failed_files)
         if (success_count or not_found_count) and not failed_files:
-            return HTTPOk(resp_body, content_type='application/json')
+            return HTTPOk(resp_body, content_type=out_content_type)
         if failed_files:
             return failed_file_response_type(
-                resp_body, content_type='application/json')
+                resp_body, content_type=out_content_type)
         return HTTPBadRequest('Invalid bulk delete.')
 
     def handle_extract(self, req, compress_type):
@@ -207,9 +290,14 @@ class Bulk(object):
         success_count = 0
         failed_files = []
         existing_containers = set()
+        out_content_type = req.accept.best_match(ACCEPTABLE_FORMATS)
+        if not out_content_type:
+            return HTTPNotAcceptable(request=req)
+        if req.content_length is None and \
+                req.headers.get('transfer-encoding', '').lower() != 'chunked':
+            return HTTPLengthRequired(request=req)
         try:
-            vrs, account, extract_base = split_path(
-                unquote(req.path), 2, 3, True)
+            vrs, account, extract_base = req.split_path(2, 3, True)
         except ValueError:
             return HTTPNotFound(request=req)
         extract_base = extract_base or ''
@@ -240,6 +328,11 @@ class Bulk(object):
                             [quote(destination[:MAX_PATH_LENGTH]),
                              HTTPPreconditionFailed().status])
                         continue
+                    if tar_info.size > MAX_FILE_SIZE:
+                        failed_files.append([
+                            quote(destination[:MAX_PATH_LENGTH]),
+                            HTTPRequestEntityTooLarge().status])
+                        continue
                     if container not in existing_containers:
                         try:
                             self.create_container(
@@ -248,12 +341,14 @@ class Bulk(object):
                         except CreateContainerError, err:
                             if err.status_int == HTTP_UNAUTHORIZED:
                                 return HTTPUnauthorized(request=req)
-                            failed_files.append([destination[:MAX_PATH_LENGTH],
-                                                 err.status])
+                            failed_files.append([
+                                quote(destination[:MAX_PATH_LENGTH]),
+                                err.status])
                             continue
                         except ValueError:
-                            failed_files.append([destination[:MAX_PATH_LENGTH],
-                                                 HTTP_BAD_REQUEST])
+                            failed_files.append([
+                                quote(destination[:MAX_PATH_LENGTH]),
+                                HTTP_BAD_REQUEST])
                             continue
                         if len(existing_containers) > self.max_containers:
                             return HTTPBadRequest(
@@ -265,6 +360,9 @@ class Bulk(object):
                     new_env['wsgi.input'] = tar_file
                     new_env['PATH_INFO'] = destination
                     new_env['CONTENT_LENGTH'] = tar_info.size
+                    new_env['swift.source'] = 'EA'
+                    new_env['HTTP_USER_AGENT'] = \
+                        '%s BulkExpand' % req.environ.get('HTTP_USER_AGENT')
                     create_obj_req = Request.blank(destination, new_env)
                     resp = create_obj_req.get_response(self.app)
                     if resp.status_int // 100 == 2:
@@ -272,42 +370,37 @@ class Bulk(object):
                     else:
                         if resp.status_int == HTTP_UNAUTHORIZED:
                             return HTTPUnauthorized(request=req)
-                        failed_files.append([destination[:MAX_PATH_LENGTH],
-                                             resp.status])
+                        failed_files.append([
+                            quote(destination[:MAX_PATH_LENGTH]), resp.status])
 
-            resp_body = json.dumps(
-                {'Number Files Created': success_count,
-                 'Errors': failed_files})
+            resp_body = get_response_body(
+                out_content_type,
+                {'Number Files Created': success_count},
+                failed_files)
             if success_count and not failed_files:
-                return HTTPCreated(resp_body, content_type='application/json')
+                return HTTPCreated(resp_body, content_type=out_content_type)
             if failed_files:
-                return HTTPBadGateway(
-                    resp_body, content_type='application/json')
+                return HTTPBadGateway(resp_body, content_type=out_content_type)
             return HTTPBadRequest('Invalid Tar File: No Valid Files')
 
         except tarfile.TarError, tar_error:
             return HTTPBadRequest('Invalid Tar File: %s' % tar_error)
 
-    def __call__(self, env, start_response):
-        req = Request(env)
-        extract_type = \
-            req.headers.get('X-Extract-Archive', '').lower().strip('.')
-        try:
-            if extract_type and req.method == 'PUT':
-                archive_type = {'tar': '', 'tar.gz': 'gz',
-                                'tar.bz2': 'bz2'}.get(extract_type)
-                if archive_type is not None:
-                    resp = self.handle_extract(req, archive_type)
-                else:
-                    resp = HTTPBadRequest("Unsupported archive format")
-                return resp(env, start_response)
-            if (req.headers.get('X-Bulk-Delete', '').lower() in TRUE_VALUES and
-                    req.method == 'DELETE'):
-                return self.handle_delete(req)(env, start_response)
-        except HTTPException, err_resp:
-            return err_resp(env, start_response)
+    @wsgify
+    def __call__(self, req):
+        extract_type = req.params.get('extract-archive')
+        if extract_type is not None and req.method == 'PUT':
+            archive_type = {
+                'tar': '', 'tar.gz': 'gz',
+                'tar.bz2': 'bz2'}.get(extract_type.lower().strip('.'))
+            if archive_type is not None:
+                return self.handle_extract(req, archive_type)
+            else:
+                return HTTPBadRequest("Unsupported archive format")
+        if 'bulk-delete' in req.params and req.method == 'DELETE':
+            return self.handle_delete(req)
 
-        return self.app(env, start_response)
+        return self.app
 
 
 def filter_factory(global_conf, **local_conf):
