@@ -16,12 +16,11 @@
 import bisect
 import itertools
 import math
+import random
 import cPickle as pickle
 
-
 from array import array
 from collections import defaultdict
-from random import randint, shuffle
 from time import time
 
 from swift.common import exceptions
@@ -40,12 +39,22 @@ class RingBuilder(object):
     a rebalance request is an isolated request or due to added, changed, or
     removed devices.
 
-    :param part_power: number of partitions = 2**part_power
+    :param part_power: number of partitions = 2**part_power.
     :param replicas: number of replicas for each partition
     :param min_part_hours: minimum number of hours between partition changes
     """
 
     def __init__(self, part_power, replicas, min_part_hours):
+        if part_power > 32:
+            raise ValueError("part_power must be at most 32 (was %d)"
+                             % (part_power,))
+        if replicas < 1:
+            raise ValueError("replicas must be at least 1 (was %.6f)"
+                             % (replicas,))
+        if min_part_hours < 0:
+            raise ValueError("min_part_hours must be non-negative (was %d)"
+                             % (min_part_hours,))
+
         self.part_power = part_power
         self.replicas = replicas
         self.min_part_hours = min_part_hours
@@ -130,6 +139,11 @@ class RingBuilder(object):
             self._remove_devs = builder['_remove_devs']
         self._ring = None
 
+        # Old builders may not have a region defined for their devices, in
+        # which case we default it to 1.
+        for dev in self._iter_devs():
+            dev.setdefault("region", 1)
+
     def to_dict(self):
         """
         Returns a dict that can be used later with copy_from to
@@ -167,6 +181,23 @@ class RingBuilder(object):
         """
         self.min_part_hours = min_part_hours
 
+    def set_replicas(self, new_replica_count):
+        """
+        Changes the number of replicas in this ring.
+
+        If the new replica count is sufficiently different that
+        self._replica2part2dev will change size, sets
+        self.devs_changed. This is so tools like
+        bin/swift-ring-builder can know to write out the new ring
+        rather than bailing out due to lack of balance change.
+        """
+        old_slots_used = int(self.parts * self.replicas)
+        new_slots_used = int(self.parts * new_replica_count)
+        if old_slots_used != new_slots_used:
+            self.devs_changed = True
+
+        self.replicas = new_replica_count
+
     def get_ring(self):
         """
         Get the ring, or more specifically, the swift.common.ring.RingData.
@@ -208,9 +239,10 @@ class RingBuilder(object):
         weight  a float of the relative weight of this device as compared to
                 others; this indicates how many partitions the builder will try
                 to assign to this device
+        region  integer indicating which region the device is in
         zone    integer indicating which zone the device is in; a given
                 partition will not be assigned to multiple devices within the
-                same zone
+                same (region, zone) pair if there is any alternative
         ip      the ip address of the device
         port    the tcp port of the device
         device  the device's name on disk (sdb1, for example)
@@ -276,7 +308,7 @@ class RingBuilder(object):
         self.devs_changed = True
         self.version += 1
 
-    def rebalance(self):
+    def rebalance(self, seed=None):
         """
         Rebalance the ring.
 
@@ -294,6 +326,10 @@ class RingBuilder(object):
 
         :returns: (number_of_partitions_altered, resulting_balance)
         """
+
+        if seed:
+            random.seed(seed)
+
         self._ring = None
         if self._last_part_moves_epoch is None:
             self._initial_balance()
@@ -302,6 +338,10 @@ class RingBuilder(object):
         retval = 0
         self._update_last_part_moves()
         last_balance = 0
+        new_parts, removed_part_count = self._adjust_replica2part2dev_size()
+        retval += removed_part_count
+        self._reassign_parts(new_parts)
+        retval += len(new_parts)
         while True:
             reassign_parts = self._gather_reassign_parts()
             self._reassign_parts(reassign_parts)
@@ -335,14 +375,15 @@ class RingBuilder(object):
         :raises RingValidationError: problem was found with the ring.
         """
 
-        # "len" showed up in profling, so it's just computed once.
+        # "len" showed up in profiling, so it's just computed once.
         dev_len = len(self.devs)
-        if sum(d['parts'] for d in self._iter_devs()) != \
-                self.parts * self.replicas:
+
+        parts_on_devs = sum(d['parts'] for d in self._iter_devs())
+        parts_in_map = sum(len(p2d) for p2d in self._replica2part2dev)
+        if parts_on_devs != parts_in_map:
             raise exceptions.RingValidationError(
                 'All partitions are not double accounted for: %d != %d' %
-                (sum(d['parts'] for d in self._iter_devs()),
-                 self.parts * self.replicas))
+                (parts_on_devs, parts_in_map))
         if stats:
             # dev_usage[dev_id] will equal the number of partitions assigned to
             # that device.
@@ -351,14 +392,19 @@ class RingBuilder(object):
                 for dev_id in part2dev:
                     dev_usage[dev_id] += 1
 
-        for part in xrange(self.parts):
-            for replica in xrange(self.replicas):
-                dev_id = self._replica2part2dev[replica][part]
-                if dev_id >= dev_len or not self.devs[dev_id]:
-                    raise exceptions.RingValidationError(
-                        "Partition %d, replica %d was not allocated "
-                        "to a device." %
-                        (part, replica))
+        for part, replica in self._each_part_replica():
+            dev_id = self._replica2part2dev[replica][part]
+            if dev_id >= dev_len or not self.devs[dev_id]:
+                raise exceptions.RingValidationError(
+                    "Partition %d, replica %d was not allocated "
+                    "to a device." %
+                    (part, replica))
+
+        for dev in self._iter_devs():
+            if not isinstance(dev['port'], int):
+                raise exceptions.RingValidationError(
+                    "Device %d has port %r, which is not an integer." %
+                    (dev['id'], dev['port']))
 
         if stats:
             weight_of_one_part = self.weight_of_one_part()
@@ -383,11 +429,11 @@ class RingBuilder(object):
     def get_balance(self):
         """
         Get the balance of the ring. The balance value is the highest
-        percentage off the desired amount of partitions a given device wants.
-        For instance, if the "worst" device wants (based on its relative weight
-        and its zone's relative weight) 123 partitions and it has 124
-        partitions, the balance value would be 0.83 (1 extra / 123 wanted * 100
-        for percentage).
+        percentage off the desired amount of partitions a given device
+        wants. For instance, if the "worst" device wants (based on its
+        weight relative to the sum of all the devices' weights) 123
+        partitions and it has 124 partitions, the balance value would
+        be 0.83 (1 extra / 123 wanted * 100 for percentage).
 
         :returns: balance of the ring
         """
@@ -419,12 +465,17 @@ class RingBuilder(object):
 
     def get_part_devices(self, part):
         """
-        Get the devices that are responsible for the partition.
+        Get the devices that are responsible for the partition,
+        filtering out duplicates.
 
         :param part: partition to get devices for
         :returns: list of device dicts
         """
-        return [self.devs[r[part]] for r in self._replica2part2dev]
+        devices = []
+        for dev in self._devs_for_part(part):
+            if dev not in devices:
+                devices.append(dev)
+        return devices
 
     def _iter_devs(self):
         """
@@ -457,19 +508,83 @@ class RingBuilder(object):
                 dev['parts_wanted'] = \
                     int(weight_of_one_part * dev['weight']) - dev['parts']
 
+    def _adjust_replica2part2dev_size(self):
+        """
+        Make sure that the lengths of the arrays in _replica2part2dev
+        are correct for the current value of self.replicas.
+
+        Example:
+        self.part_power = 8
+        self.replicas = 2.25
+
+        self._replica2part2dev will contain 3 arrays: the first 2 of
+        length 256 (2**8), and the last of length 64 (0.25 * 2**8).
+
+        Returns a 2-tuple: the first element is a list of (partition,
+        replicas) tuples indicating which replicas need to be
+        (re)assigned to devices, and the second element is a count of
+        how many replicas were removed.
+        """
+        removed_replicas = 0
+
+        fractional_replicas, whole_replicas = math.modf(self.replicas)
+        whole_replicas = int(whole_replicas)
+
+        desired_lengths = [self.parts] * whole_replicas
+        if fractional_replicas:
+            desired_lengths.append(int(self.parts * fractional_replicas))
+
+        to_assign = defaultdict(list)
+
+        if self._replica2part2dev is not None:
+            # If we crossed an integer threshold (say, 4.1 --> 4),
+            # we'll have a partial extra replica clinging on here. Clean
+            # up any such extra stuff.
+            for part2dev in self._replica2part2dev[len(desired_lengths):]:
+                for dev_id in part2dev:
+                    dev_losing_part = self.devs[dev_id]
+                    dev_losing_part['parts'] -= 1
+                    removed_replicas += 1
+            self._replica2part2dev = \
+                self._replica2part2dev[:len(desired_lengths)]
+        else:
+            self._replica2part2dev = []
+
+        for replica, desired_length in enumerate(desired_lengths):
+            if replica < len(self._replica2part2dev):
+                part2dev = self._replica2part2dev[replica]
+                if len(part2dev) < desired_length:
+                    # Not long enough: needs to be extended and the
+                    # newly-added pieces assigned to devices.
+                    for part in xrange(len(part2dev), desired_length):
+                        to_assign[part].append(replica)
+                        part2dev.append(0)
+                elif len(part2dev) > desired_length:
+                    # Too long: truncate this mapping.
+                    for part in xrange(desired_length, len(part2dev)):
+                        dev_losing_part = self.devs[part2dev[part]]
+                        dev_losing_part['parts'] -= 1
+                        removed_replicas += 1
+                    self._replica2part2dev[replica] = part2dev[:desired_length]
+            else:
+                # Mapping not present at all: make one up and assign
+                # all of it.
+                for part in xrange(desired_length):
+                    to_assign[part].append(replica)
+                self._replica2part2dev.append(
+                    array('H', (0 for _junk in xrange(desired_length))))
+
+        return (list(to_assign.iteritems()), removed_replicas)
+
     def _initial_balance(self):
         """
         Initial partition assignment is the same as rebalancing an
         existing ring, but with some initial setup beforehand.
         """
-        self._replica2part2dev = \
-            [array('H', (0 for _junk in xrange(self.parts)))
-             for _junk in xrange(self.replicas)]
-
-        replicas = range(self.replicas)
         self._last_part_moves = array('B', (0 for _junk in xrange(self.parts)))
         self._last_part_moves_epoch = int(time())
-        self._reassign_parts((p, replicas) for p in xrange(self.parts))
+
+        self._reassign_parts(self._adjust_replica2part2dev_size()[0])
 
     def _update_last_part_moves(self):
         """
@@ -506,12 +621,11 @@ class RingBuilder(object):
         if self._remove_devs:
             dev_ids = [d['id'] for d in self._remove_devs if d['parts']]
             if dev_ids:
-                for replica in xrange(self.replicas):
-                    part2dev = self._replica2part2dev[replica]
-                    for part in xrange(self.parts):
-                        if part2dev[part] in dev_ids:
-                            self._last_part_moves[part] = 0
-                            removed_dev_parts[part].append(replica)
+                for part, replica in self._each_part_replica():
+                    dev_id = self._replica2part2dev[replica][part]
+                    if dev_id in dev_ids:
+                        self._last_part_moves[part] = 0
+                        removed_dev_parts[part].append(replica)
 
         # Now we gather partitions that are "at risk" because they aren't
         # currently sufficient spread out across the cluster.
@@ -527,8 +641,7 @@ class RingBuilder(object):
             # replicas_at_tier was a "lambda: 0" defaultdict, but profiling
             # revealed the lambda invocation as a significant cost.
             replicas_at_tier = {}
-            for replica in xrange(self.replicas):
-                dev = self.devs[self._replica2part2dev[replica][part]]
+            for dev in self._devs_for_part(part):
                 if dev['id'] not in tfd:
                     tfd[dev['id']] = tiers_for_dev(dev)
                 for tier in tfd[dev['id']]:
@@ -539,7 +652,7 @@ class RingBuilder(object):
 
             # Now, look for partitions not yet spread out enough and not
             # recently moved.
-            for replica in xrange(self.replicas):
+            for replica in self._replicas_for_part(part):
                 dev = self.devs[self._replica2part2dev[replica][part]]
                 removed_replica = False
                 if dev['id'] not in tfd:
@@ -570,12 +683,19 @@ class RingBuilder(object):
         # We randomly pick a new starting point in the "circular" ring of
         # partitions to try to get a better rebalance when called multiple
         # times.
-        start = self._last_part_gather_start / 4 + randint(0, self.parts / 2)
+
+        start = self._last_part_gather_start / 4
+        start += random.randint(0, self.parts / 2)  # GRAH PEP8!!!
+
         self._last_part_gather_start = start
-        for replica in xrange(self.replicas):
-            part2dev = self._replica2part2dev[replica]
-            for part in itertools.chain(xrange(start, self.parts),
-                                        xrange(0, start)):
+        for replica, part2dev in enumerate(self._replica2part2dev):
+            # If we've got a partial replica, start may be out of
+            # range. Scale it down so that we get a similar movement
+            # pattern (but scaled down) on sequential runs.
+            this_start = int(float(start) * len(part2dev) / self.parts)
+
+            for part in itertools.chain(xrange(this_start, len(part2dev)),
+                                        xrange(0, this_start)):
                 if self._last_part_moves[part] < self.min_part_hours:
                     continue
                 if part in removed_dev_parts or part in spread_out_parts:
@@ -598,7 +718,7 @@ class RingBuilder(object):
         # it would concentrate load during failure recovery scenarios
         # (increasing risk). The "right" answer has yet to be debated to
         # conclusion, but working code wins for now.
-        shuffle(reassign_parts_list)
+        random.shuffle(reassign_parts_list)
         return reassign_parts_list
 
     def _reassign_parts(self, reassign_parts):
@@ -608,10 +728,10 @@ class RingBuilder(object):
         they still want and kept in that order throughout the process. The
         gathered partitions are iterated through, assigning them to devices
         according to the "most wanted" while keeping the replicas as "far
-        apart" as possible. Two different zones are considered the
-        farthest-apart things, followed by different ip/port pairs within a
-        zone; the least-far-apart things are different devices with the same
-        ip/port pair in the same zone.
+        apart" as possible. Two different regions are considered the
+        farthest-apart things, followed by zones, then different ip/port pairs
+        within a zone; the least-far-apart things are different devices with
+        the same ip/port pair in the same zone.
 
         If you want more replicas than devices, you won't get all your
         replicas.
@@ -624,69 +744,102 @@ class RingBuilder(object):
         """
         for dev in self._iter_devs():
             dev['sort_key'] = self._sort_key_for(dev)
+
         available_devs = \
             sorted((d for d in self._iter_devs() if d['weight']),
                    key=lambda x: x['sort_key'])
 
-        tier2children = build_tier_tree(available_devs)
-
         tier2devs = defaultdict(list)
         tier2sort_key = defaultdict(list)
-        tiers_by_depth = defaultdict(set)
+        max_tier_depth = 0
         for dev in available_devs:
             for tier in tiers_for_dev(dev):
                 tier2devs[tier].append(dev)  # <-- starts out sorted!
                 tier2sort_key[tier].append(dev['sort_key'])
-                tiers_by_depth[len(tier)].add(tier)
+                if len(tier) > max_tier_depth:
+                    max_tier_depth = len(tier)
+
+        tier2children_sets = build_tier_tree(available_devs)
+        tier2children = defaultdict(list)
+        tier2children_sort_key = {}
+        tiers_list = [()]
+        depth = 1
+        while depth <= max_tier_depth:
+            new_tiers_list = []
+            for tier in tiers_list:
+                child_tiers = list(tier2children_sets[tier])
+                child_tiers.sort(key=lambda t: tier2sort_key[t][-1])
+                tier2children[tier] = child_tiers
+                tier2children_sort_key[tier] = map(
+                    lambda t: tier2sort_key[t][-1], child_tiers)
+                new_tiers_list.extend(child_tiers)
+            tiers_list = new_tiers_list
+            depth += 1
 
         for part, replace_replicas in reassign_parts:
-            # Gather up what other tiers (zones, ip_ports, and devices) the
-            # replicas not-to-be-moved are in for this part.
-            other_replicas = defaultdict(lambda: 0)
-            for replica in xrange(self.replicas):
+            # Gather up what other tiers (regions, zones, ip/ports, and
+            # devices) the replicas not-to-be-moved are in for this part.
+            other_replicas = defaultdict(int)
+            unique_tiers_by_tier_len = defaultdict(set)
+            for replica in self._replicas_for_part(part):
                 if replica not in replace_replicas:
                     dev = self.devs[self._replica2part2dev[replica][part]]
                     for tier in tiers_for_dev(dev):
                         other_replicas[tier] += 1
-
-            def find_home_for_replica(tier=(), depth=1):
-                # Order the tiers by how many replicas of this
-                # partition they already have. Then, of the ones
-                # with the smallest number of replicas, pick the
-                # tier with the hungriest drive and then continue
-                # searching in that subtree.
-                #
-                # There are other strategies we could use here,
-                # such as hungriest-tier (i.e. biggest
-                # sum-of-parts-wanted) or picking one at random.
-                # However, hungriest-drive is what was used here
-                # before, and it worked pretty well in practice.
-                #
-                # Note that this allocator will balance things as
-                # evenly as possible at each level of the device
-                # layout. If your layout is extremely unbalanced,
-                # this may produce poor results.
-                candidate_tiers = tier2children[tier]
-                min_count = min(other_replicas[t] for t in candidate_tiers)
-                candidate_tiers = [t for t in candidate_tiers
-                                   if other_replicas[t] == min_count]
-                candidate_tiers.sort(
-                    key=lambda t: tier2sort_key[t][-1])
-
-                if depth == max(tiers_by_depth.keys()):
-                    return tier2devs[candidate_tiers[-1]][-1]
-
-                return find_home_for_replica(tier=candidate_tiers[-1],
-                                             depth=depth + 1)
+                        unique_tiers_by_tier_len[len(tier)].add(tier)
 
             for replica in replace_replicas:
-                dev = find_home_for_replica()
+                tier = ()
+                depth = 1
+                while depth <= max_tier_depth:
+                    # Order the tiers by how many replicas of this
+                    # partition they already have. Then, of the ones
+                    # with the smallest number of replicas, pick the
+                    # tier with the hungriest drive and then continue
+                    # searching in that subtree.
+                    #
+                    # There are other strategies we could use here,
+                    # such as hungriest-tier (i.e. biggest
+                    # sum-of-parts-wanted) or picking one at random.
+                    # However, hungriest-drive is what was used here
+                    # before, and it worked pretty well in practice.
+                    #
+                    # Note that this allocator will balance things as
+                    # evenly as possible at each level of the device
+                    # layout. If your layout is extremely unbalanced,
+                    # this may produce poor results.
+                    #
+                    # This used to be a cute, recursive function, but it's been
+                    # unrolled for performance.
+                    candidate_tiers = tier2children[tier]
+                    candidates_with_replicas = \
+                        unique_tiers_by_tier_len[len(tier) + 1]
+                    if len(candidate_tiers) > len(candidates_with_replicas):
+                        # There exists at least one tier with 0 other replicas,
+                        # so work backward among the candidates, accepting the
+                        # first which isn't in other_replicas.
+                        #
+                        # This optimization is to avoid calling the min()
+                        # below, which is expensive if you've got thousands of
+                        # drives.
+                        for t in reversed(candidate_tiers):
+                            if other_replicas[t] == 0:
+                                tier = t
+                                break
+                    else:
+                        min_count = min(other_replicas[t]
+                                        for t in candidate_tiers)
+                        tier = (t for t in reversed(candidate_tiers)
+                                if other_replicas[t] == min_count).next()
+                    depth += 1
+                dev = tier2devs[tier][-1]
                 dev['parts_wanted'] -= 1
                 dev['parts'] += 1
                 old_sort_key = dev['sort_key']
                 new_sort_key = dev['sort_key'] = self._sort_key_for(dev)
                 for tier in tiers_for_dev(dev):
                     other_replicas[tier] += 1
+                    unique_tiers_by_tier_len[len(tier)].add(tier)
 
                     index = bisect.bisect_left(tier2sort_key[tier],
                                                old_sort_key)
@@ -698,6 +851,22 @@ class RingBuilder(object):
                     tier2devs[tier].insert(new_index, dev)
                     tier2sort_key[tier].insert(new_index, new_sort_key)
 
+                    # Now jiggle tier2children values to keep them sorted
+                    new_last_sort_key = tier2sort_key[tier][-1]
+                    parent_tier = tier[0:-1]
+                    index = bisect.bisect_left(
+                        tier2children_sort_key[parent_tier],
+                        old_sort_key)
+                    popped = tier2children[parent_tier].pop(index)
+                    tier2children_sort_key[parent_tier].pop(index)
+
+                    new_index = bisect.bisect_left(
+                        tier2children_sort_key[parent_tier],
+                        new_last_sort_key)
+                    tier2children[parent_tier].insert(new_index, popped)
+                    tier2children_sort_key[parent_tier].insert(
+                        new_index, new_last_sort_key)
+
                 self._replica2part2dev[replica][part] = dev['id']
 
         # Just to save memory and keep from accidental reuse.
@@ -714,7 +883,7 @@ class RingBuilder(object):
         # parts_wanted end up sorted above positive parts_wanted.
         return '%016x.%04x.%04x' % (
             (self.parts * self.replicas) + dev['parts_wanted'],
-            randint(0, 0xffff),
+            random.randint(0, 0xFFFF),
             dev['id'])
 
     def _build_max_replicas_by_tier(self):
@@ -772,6 +941,37 @@ class RingBuilder(object):
             return mr
         return walk_tree((), self.replicas)
 
+    def _devs_for_part(self, part):
+        """
+        Returns a list of devices for a specified partition.
+
+        Deliberately includes duplicates.
+        """
+        if self._replica2part2dev is None:
+            return []
+        return [self.devs[part2dev[part]]
+                for part2dev in self._replica2part2dev
+                if part < len(part2dev)]
+
+    def _replicas_for_part(self, part):
+        """
+        Returns a list of replicas for a specified partition.
+
+        These can be used as indices into self._replica2part2dev
+        without worrying about IndexErrors.
+        """
+        return [replica for replica, part2dev
+                in enumerate(self._replica2part2dev)
+                if part < len(part2dev)]
+
+    def _each_part_replica(self):
+        """
+        Generator yielding every (partition, replica) pair in the ring.
+        """
+        for replica, part2dev in enumerate(self._replica2part2dev):
+            for part in xrange(len(part2dev)):
+                yield (part, replica)
+
     @classmethod
     def load(cls, builder_file, open=open):
         """
@@ -793,11 +993,16 @@ class RingBuilder(object):
 
     def search_devs(self, search_value):
         """
-The <search-value> can be of the form:
-    d<device_id>z<zone>-<ip>:<port>/<device_name>_<meta>
+    The <search-value> can be of the form::
+
+        d<device_id>r<region>z<zone>-<ip>:<port>/<device_name>_<meta>
+
     Any part is optional, but you must include at least one part.
-    Examples:
+
+    Examples::
+
         d74              Matches the device id 74
+        r4               Matches devices in region 4
         z1               Matches devices in zone 1
         z1-1.2.3.4       Matches devices in zone 1 with the ip 1.2.3.4
         1.2.3.4          Matches devices in any zone with the ip 1.2.3.4
@@ -808,9 +1013,13 @@ The <search-value> can be of the form:
         _"snet: 5.6.7.8" Matches devices with snet: 5.6.7.8 in the meta data
         [::1]            Matches devices in any zone with the ip ::1
         z1-[::1]:5678    Matches devices in zone 1 with ip ::1 and port 5678
-    Most specific example:
-        d74z1-1.2.3.4:5678/sdb1_"snet: 5.6.7.8"
+
+    Most specific example::
+
+        d74r4z1-1.2.3.4:5678/sdb1_"snet: 5.6.7.8"
+
     Nerd explanation:
+
         All items require their single character prefix except the ip, in which
         case the - is optional unless the device id or zone is also included.
         """
@@ -822,6 +1031,12 @@ The <search-value> can be of the form:
                 i += 1
             match.append(('id', int(search_value[1:i])))
             search_value = search_value[i:]
+        if search_value.startswith('r'):
+            i = 1
+            while i < len(search_value) and search_value[i].isdigit():
+                i += 1
+            match.append(('region', int(search_value[1:i])))
+            search_value = search_value[i:]
         if search_value.startswith('z'):
             i = 1
             while i < len(search_value) and search_value[i].isdigit():
