@@ -22,15 +22,12 @@ import struct
 from time import time
 import os
 from io import BufferedReader
+from hashlib import md5
+from itertools import chain
 
-from swift.common.utils import hash_path, validate_configuration
+from swift.common.utils import hash_path, validate_configuration, json
 from swift.common.ring.utils import tiers_for_dev
 
-try:
-    import simplejson as json
-except ImportError:
-    import json
-
 
 class RingData(object):
     """Partitioned consistent hashing ring data (used for serialization)."""
@@ -40,6 +37,10 @@ class RingData(object):
         self._replica2part2dev_id = replica2part2dev_id
         self._part_shift = part_shift
 
+        for dev in self.devs:
+            if dev is not None:
+                dev.setdefault("region", 1)
+
     @classmethod
     def deserialize_v1(cls, gz_file):
         json_len, = struct.unpack('!I', gz_file.read(4))
@@ -85,7 +86,8 @@ class RingData(object):
         # Write out new-style serialization magic and version:
         file_obj.write(struct.pack('!4sH', 'R1NG', 1))
         ring = self.to_dict()
-        json_text = json.dumps(
+        json_encoder = json.JSONEncoder(sort_keys=True)
+        json_text = json_encoder.encode(
             {'devs': ring['devs'], 'part_shift': ring['part_shift'],
              'replica_count': len(ring['replica2part2dev_id'])})
         json_len = len(json_text)
@@ -100,7 +102,16 @@ class RingData(object):
 
         :param filename: File into which this instance should be serialized.
         """
-        gz_file = GzipFile(filename, 'wb')
+        # Override the timestamp so that the same ring data creates
+        # the same bytes on disk. This makes a checksum comparison a
+        # good way to see if two rings are identical.
+        #
+        # This only works on Python 2.7; on 2.6, we always get the
+        # current time in the gzip output.
+        try:
+            gz_file = GzipFile(filename, 'wb', mtime=1300507380.0)
+        except TypeError:
+            gz_file = GzipFile(filename, 'wb')
         self.serialize_v1(gz_file)
         gz_file.close()
 
@@ -158,7 +169,7 @@ class Ring(object):
 
     @property
     def replica_count(self):
-        """Number of replicas used in the ring."""
+        """Number of replicas (full or partial) used in the ring."""
         return len(self._replica2part2dev_id)
 
     @property
@@ -183,9 +194,15 @@ class Ring(object):
         return getmtime(self.serialized_path) != self._mtime
 
     def _get_part_nodes(self, part):
+        part_nodes = []
         seen_ids = set()
-        return [self._devs[r[part]] for r in self._replica2part2dev_id
-                if not (r[part] in seen_ids or seen_ids.add(r[part]))]
+        for r2p2d in self._replica2part2dev_id:
+            if part < len(r2p2d):
+                dev_id = r2p2d[part]
+                if dev_id not in seen_ids:
+                    part_nodes.append(self.devs[dev_id])
+                seen_ids.add(dev_id)
+        return part_nodes
 
     def get_part_nodes(self, part):
         """
@@ -241,6 +258,11 @@ class Ring(object):
         """
         Generator to get extra nodes for a partition for hinted handoff.
 
+        The handoff nodes will try to be in zones other than the
+        primary zones, will take into account the device weights, and
+        will usually keep the same sequences of handoffs even with
+        ring changes.
+
         :param part: partition to get handoff nodes for
         :returns: generator of node dicts
 
@@ -248,22 +270,52 @@ class Ring(object):
         """
         if time() > self._rtime:
             self._reload()
-        used_tiers = set()
-        for part2dev_id in self._replica2part2dev_id:
-            for tier in tiers_for_dev(self._devs[part2dev_id[part]]):
-                used_tiers.add(tier)
+        primary_nodes = self._get_part_nodes(part)
 
-        for level in self.tiers_by_length:
-            tiers = list(level)
-            while tiers:
-                tier = tiers.pop(part % len(tiers))
-                if tier in used_tiers:
-                    continue
-                for i in xrange(len(self.tier2devs[tier])):
-                    dev = self.tier2devs[tier][(part + i) %
-                                               len(self.tier2devs[tier])]
-                    if not dev.get('weight'):
-                        continue
-                    yield dev
-                    used_tiers.update(tiers_for_dev(dev))
-                    break
+        used = set(d['id'] for d in primary_nodes)
+        same_regions = set(d['region'] for d in primary_nodes)
+        same_zones = set((d['region'], d['zone']) for d in primary_nodes)
+
+        parts = len(self._replica2part2dev_id[0])
+        start = struct.unpack_from(
+            '>I', md5(str(part)).digest())[0] >> self._part_shift
+        inc = int(parts / 65536) or 1
+        # Multiple loops for execution speed; the checks and bookkeeping get
+        # simpler as you go along
+        for handoff_part in chain(xrange(start, parts, inc),
+                                  xrange(inc - ((parts - start) % inc),
+                                         start, inc)):
+            for part2dev_id in self._replica2part2dev_id:
+                if handoff_part < len(part2dev_id):
+                    dev_id = part2dev_id[handoff_part]
+                    dev = self._devs[dev_id]
+                    region = dev['region']
+                    zone = (dev['region'], dev['zone'])
+                    if dev_id not in used and region not in same_regions:
+                        yield dev
+                        used.add(dev_id)
+                        same_regions.add(region)
+                        same_zones.add(zone)
+
+        for handoff_part in chain(xrange(start, parts, inc),
+                                  xrange(inc - ((parts - start) % inc),
+                                         start, inc)):
+            for part2dev_id in self._replica2part2dev_id:
+                if handoff_part < len(part2dev_id):
+                    dev_id = part2dev_id[handoff_part]
+                    dev = self._devs[dev_id]
+                    zone = (dev['region'], dev['zone'])
+                    if dev_id not in used and zone not in same_zones:
+                        yield dev
+                        used.add(dev_id)
+                        same_zones.add(zone)
+
+        for handoff_part in chain(xrange(start, parts, inc),
+                                  xrange(inc - ((parts - start) % inc),
+                                         start, inc)):
+            for part2dev_id in self._replica2part2dev_id:
+                if handoff_part < len(part2dev_id):
+                    dev_id = part2dev_id[handoff_part]
+                    if dev_id not in used:
+                        yield self._devs[dev_id]
+                        used.add(dev_id)
