@@ -31,7 +31,7 @@ from xattr import getxattr, setxattr
 from eventlet import sleep, Timeout, tpool
 
 from swift.common.utils import mkdirs, normalize_timestamp, public, \
-    storage_directory, hash_path, renamer, fallocate, fsync, \
+    storage_directory, hash_path, renamer, fallocate, fsync, fdatasync, \
     split_path, drop_buffer_cache, get_logger, write_pickle, \
     config_true_value, validate_device_partition, timing_stats
 from swift.common.bufferedhttp import http_connect
@@ -207,7 +207,7 @@ class DiskFile(object):
 
     def app_iter_ranges(self, ranges, content_type, boundary, size):
         """Returns an iterator over the data file for a set of ranges"""
-        if (not ranges or len(ranges) == 0):
+        if not ranges:
             yield ''
         else:
             try:
@@ -223,8 +223,8 @@ class DiskFile(object):
     def _handle_close_quarantine(self):
         """Check if file needs to be quarantined"""
         try:
-            obj_size = self.get_data_file_size()
-        except DiskFileError, e:
+            self.get_data_file_size()
+        except DiskFileError:
             self.quarantine()
             return
         except DiskFileNotExist:
@@ -417,6 +417,7 @@ class ObjectController(object):
             content-encoding,
             x-delete-at,
             x-object-manifest,
+            x-static-large-object,
         '''
         self.allowed_headers = set(
             i.strip().lower() for i in
@@ -489,16 +490,34 @@ class ObjectController(object):
         :param obj: object name
         :param headers_in: dictionary of headers from the original request
         :param headers_out: dictionary of headers to send in the container
-                            request
+                            request(s)
         :param objdevice: device name that the object is in
         """
-        host = headers_in.get('X-Container-Host', None)
-        partition = headers_in.get('X-Container-Partition', None)
-        contdevice = headers_in.get('X-Container-Device', None)
-        if not all([host, partition, contdevice]):
+        conthosts = [h.strip() for h in
+                     headers_in.get('X-Container-Host', '').split(',')]
+        contdevices = [d.strip() for d in
+                       headers_in.get('X-Container-Device', '').split(',')]
+        contpartition = headers_in.get('X-Container-Partition', '')
+
+        if len(conthosts) != len(contdevices):
+            # This shouldn't happen unless there's a bug in the proxy,
+            # but if there is, we want to know about it.
+            self.logger.error(_('ERROR Container update failed: different  '
+                                'numbers of hosts and devices in request: '
+                                '"%s" vs "%s"' %
+                                (headers_in.get('X-Container-Host', ''),
+                                 headers_in.get('X-Container-Device', ''))))
             return
-        self.async_update(op, account, container, obj, host, partition,
-                          contdevice, headers_out, objdevice)
+
+        if contpartition:
+            updates = zip(conthosts, contdevices)
+        else:
+            updates = []
+
+        for conthost, contdevice in updates:
+            self.async_update(op, account, container, obj, conthost,
+                              contpartition, contdevice, headers_out,
+                              objdevice)
 
     def delete_at_update(self, op, delete_at, account, container, obj,
                          headers_in, objdevice):
@@ -516,25 +535,36 @@ class ObjectController(object):
         # At that time, Swift will be so popular and pervasive I will have
         # created income for thousands of future programmers.
         delete_at = max(min(delete_at, 9999999999), 0)
-        host = partition = contdevice = None
+        updates = [(None, None)]
+
+        partition = None
+        hosts = contdevices = [None]
         headers_out = {'x-timestamp': headers_in['x-timestamp'],
                        'x-trans-id': headers_in.get('x-trans-id', '-')}
         if op != 'DELETE':
-            host = headers_in.get('X-Delete-At-Host', None)
             partition = headers_in.get('X-Delete-At-Partition', None)
-            contdevice = headers_in.get('X-Delete-At-Device', None)
+            hosts = headers_in.get('X-Delete-At-Host', '')
+            contdevices = headers_in.get('X-Delete-At-Device', '')
+            updates = [upd for upd in
+                       zip((h.strip() for h in hosts.split(',')),
+                           (c.strip() for c in contdevices.split(',')))
+                       if all(upd) and partition]
+            if not updates:
+                updates = [(None, None)]
             headers_out['x-size'] = '0'
             headers_out['x-content-type'] = 'text/plain'
             headers_out['x-etag'] = 'd41d8cd98f00b204e9800998ecf8427e'
-        self.async_update(
-            op, self.expiring_objects_account,
-            str(delete_at / self.expiring_objects_container_divisor *
-                self.expiring_objects_container_divisor),
-            '%s-%s/%s/%s' % (delete_at, account, container, obj),
-            host, partition, contdevice, headers_out, objdevice)
+
+        for host, contdevice in updates:
+            self.async_update(
+                op, self.expiring_objects_account,
+                str(delete_at / self.expiring_objects_container_divisor *
+                    self.expiring_objects_container_divisor),
+                '%s-%s/%s/%s' % (delete_at, account, container, obj),
+                host, partition, contdevice, headers_out, objdevice)
 
     @public
-    @timing_stats
+    @timing_stats()
     def POST(self, request):
         """Handle HTTP POST requests for the Swift Object Server."""
         try:
@@ -560,7 +590,7 @@ class ObjectController(object):
         if file.is_deleted() or file.is_expired():
             return HTTPNotFound(request=request)
         try:
-            file_size = file.get_data_file_size()
+            file.get_data_file_size()
         except (DiskFileError, DiskFileNotExist):
             file.quarantine()
             return HTTPNotFound(request=request)
@@ -583,7 +613,7 @@ class ObjectController(object):
         return HTTPAccepted(request=request)
 
     @public
-    @timing_stats
+    @timing_stats()
     def PUT(self, request):
         """Handle HTTP PUT requests for the Swift Object Server."""
         try:
@@ -613,15 +643,15 @@ class ObjectController(object):
         etag = md5()
         upload_size = 0
         last_sync = 0
+        elapsed_time = 0
         with file.mkstemp() as fd:
-            if 'content-length' in request.headers:
-                try:
-                    fallocate(fd, int(request.headers['content-length']))
-                except OSError:
-                    return HTTPInsufficientStorage(drive=device,
-                                                   request=request)
+            try:
+                fallocate(fd, int(request.headers.get('content-length', 0)))
+            except OSError:
+                return HTTPInsufficientStorage(drive=device, request=request)
             reader = request.environ['wsgi.input'].read
             for chunk in iter(lambda: reader(self.network_chunk_size), ''):
+                start_time = time.time()
                 upload_size += len(chunk)
                 if time.time() > upload_expiration:
                     self.logger.increment('PUT.timeouts')
@@ -632,10 +662,15 @@ class ObjectController(object):
                     chunk = chunk[written:]
                 # For large files sync every 512MB (by default) written
                 if upload_size - last_sync >= self.bytes_per_sync:
-                    tpool.execute(fsync, fd)
+                    tpool.execute(fdatasync, fd)
                     drop_buffer_cache(fd, last_sync, upload_size - last_sync)
                     last_sync = upload_size
                 sleep()
+                elapsed_time += time.time() - start_time
+
+            if upload_size:
+                self.logger.transfer_rate(
+                    'PUT.' + device + '.timing', elapsed_time, upload_size)
 
             if 'content-length' in request.headers and \
                     int(request.headers['content-length']) != upload_size:
@@ -683,7 +718,7 @@ class ObjectController(object):
         return resp
 
     @public
-    @timing_stats
+    @timing_stats()
     def GET(self, request):
         """Handle HTTP GET requests for the Swift Object Server."""
         try:
@@ -763,7 +798,7 @@ class ObjectController(object):
         return request.get_response(response)
 
     @public
-    @timing_stats
+    @timing_stats(sample_rate=0.8)
     def HEAD(self, request):
         """Handle HTTP HEAD requests for the Swift Object Server."""
         try:
@@ -803,7 +838,7 @@ class ObjectController(object):
         return response
 
     @public
-    @timing_stats
+    @timing_stats()
     def DELETE(self, request):
         """Handle HTTP DELETE requests for the Swift Object Server."""
         try:
@@ -851,7 +886,7 @@ class ObjectController(object):
         return resp
 
     @public
-    @timing_stats
+    @timing_stats(sample_rate=0.1)
     def REPLICATE(self, request):
         """
         Handle REPLICATE requests for the Swift Object Server.  This is used
