@@ -20,6 +20,7 @@ import urllib
 from shutil import rmtree
 from tempfile import mkdtemp
 from StringIO import StringIO
+from mock import patch
 from swift.common.middleware import bulk
 from swift.common.swob import Request, Response, HTTPException
 from swift.common.utils import json
@@ -141,13 +142,34 @@ class TestUntar(unittest.TestCase):
                                mode=mode)
             tar.add(os.path.join(self.testdir, base_name))
             tar.close()
-            req = Request.blank('/tar_works/acc/cont/')
+            req = Request.blank('/tar_works/acc/cont/',
+                                headers={'Accept': 'application/json'})
             req.environ['wsgi.input'] = open(
                 os.path.join(self.testdir, 'tar_works.tar' + extension))
+            req.headers['transfer-encoding'] = 'chunked'
             resp = self.bulk.handle_extract(req, compress_format)
             resp_data = json.loads(resp.body)
             self.assertEquals(resp_data['Number Files Created'], 6)
 
+            # test out xml
+            req = Request.blank('/tar_works/acc/cont/',
+                                headers={'Accept': 'application/xml'})
+            req.environ['wsgi.input'] = open(
+                os.path.join(self.testdir, 'tar_works.tar' + extension))
+            req.headers['transfer-encoding'] = 'chunked'
+            resp = self.bulk.handle_extract(req, compress_format)
+            self.assertEquals(resp.status_int, 201)
+            self.assert_('<number_files_created>6</number_files_created>' in
+                         resp.body)
+
+            # test out nonexistent format
+            req = Request.blank('/tar_works/acc/cont/',
+                                headers={'Accept': 'good_xml'})
+            req.environ['wsgi.input'] = open(
+                os.path.join(self.testdir, 'tar_works.tar' + extension))
+            resp = self.bulk.handle_extract(req, compress_format)
+            self.assertEquals(resp.status_int, 406)
+
     def test_extract_call(self):
         base_name = 'base_works_gz'
         dir_tree = [
@@ -165,7 +187,7 @@ class TestUntar(unittest.TestCase):
         def fake_start_response(*args, **kwargs):
             pass
 
-        req = Request.blank('/tar_works/acc/cont/')
+        req = Request.blank('/tar_works/acc/cont/?extract-archive=tar.gz')
         req.environ['wsgi.input'] = open(
             os.path.join(self.testdir, 'tar_works.tar.gz'))
         self.bulk(req.environ, fake_start_response)
@@ -174,13 +196,17 @@ class TestUntar(unittest.TestCase):
         self.app.calls = 0
         req.environ['wsgi.input'] = open(
             os.path.join(self.testdir, 'tar_works.tar.gz'))
-        req.headers['x-extract-archive'] = 'tar.gz'
+        req.headers['transfer-encoding'] = 'Chunked'
         req.method = 'PUT'
         self.bulk(req.environ, fake_start_response)
         self.assertEquals(self.app.calls, 7)
 
         self.app.calls = 0
-        req.headers['x-extract-archive'] = 'bad'
+        req = Request.blank('/tar_works/acc/cont/?extract-archive=bad')
+        req.method = 'PUT'
+        req.headers['transfer-encoding'] = 'Chunked'
+        req.environ['wsgi.input'] = open(
+            os.path.join(self.testdir, 'tar_works.tar.gz'))
         t = self.bulk(req.environ, fake_start_response)
         self.assertEquals(t[0], "Unsupported archive format")
 
@@ -190,20 +216,23 @@ class TestUntar(unittest.TestCase):
         tar.add(os.path.join(self.testdir, base_name))
         tar.close()
         self.app.calls = 0
+        req = Request.blank('/tar_works/acc/cont/?extract-archive=tar')
+        req.method = 'PUT'
+        req.headers['transfer-encoding'] = 'Chunked'
         req.environ['wsgi.input'] = open(
             os.path.join(self.testdir, 'tar_works.tar'))
-        req.headers['x-extract-archive'] = 'tar'
         t = self.bulk(req.environ, fake_start_response)
         self.assertEquals(self.app.calls, 7)
 
     def test_bad_container(self):
-        req = Request.blank('/invalid/')
+        req = Request.blank('/invalid/', body='')
         resp = self.bulk.handle_extract(req, '')
         self.assertEquals(resp.status_int, 404)
 
+    def test_content_length_required(self):
         req = Request.blank('/create_cont_fail/acc/cont')
         resp = self.bulk.handle_extract(req, '')
-        self.assertEquals(resp.status_int, 400)
+        self.assertEquals(resp.status_int, 411)
 
     def build_tar(self, dir_tree=None):
         if not dir_tree:
@@ -226,9 +255,11 @@ class TestUntar(unittest.TestCase):
                              {'sub_dir2': ['sub2_file1', 'sub2_file2']},
                              {'sub_dir3': [{'sub4_dir1': 'sub4_file1'}]}]}]
         tar = self.build_tar(dir_tree)
-        req = Request.blank('/tar_works/acc/')
+        req = Request.blank('/tar_works/acc/',
+                            headers={'Accept': 'application/json'})
         req.environ['wsgi.input'] = open(os.path.join(self.testdir,
                                                       'tar_fails.tar'))
+        req.headers['transfer-encoding'] = 'chunked'
         resp = self.bulk.handle_extract(req, '')
         resp_data = json.loads(resp.body)
         self.assertEquals(resp_data['Number Files Created'], 4)
@@ -238,6 +269,7 @@ class TestUntar(unittest.TestCase):
         req = Request.blank('/unauth/acc/')
         req.environ['wsgi.input'] = open(os.path.join(self.testdir,
                                                       'tar_fails.tar'))
+        req.headers['transfer-encoding'] = 'chunked'
         resp = self.bulk.handle_extract(req, '')
         self.assertEquals(resp.status_int, 401)
 
@@ -246,14 +278,17 @@ class TestUntar(unittest.TestCase):
         req = Request.blank('/create_obj_unauth/acc/cont/')
         req.environ['wsgi.input'] = open(os.path.join(self.testdir,
                                                       'tar_fails.tar'))
+        req.headers['transfer-encoding'] = 'chunked'
         resp = self.bulk.handle_extract(req, '')
         self.assertEquals(resp.status_int, 401)
 
     def test_extract_tar_fail_obj_name_len(self):
         tar = self.build_tar()
-        req = Request.blank('/tar_works/acc/cont/')
+        req = Request.blank('/tar_works/acc/cont/',
+                            headers={'Accept': 'application/json'})
         req.environ['wsgi.input'] = open(os.path.join(self.testdir,
                                                       'tar_fails.tar'))
+        req.headers['transfer-encoding'] = 'chunked'
         resp = self.bulk.handle_extract(req, '')
         resp_data = json.loads(resp.body)
         self.assertEquals(resp_data['Number Files Created'], 4)
@@ -265,26 +300,45 @@ class TestUntar(unittest.TestCase):
         req = Request.blank('/tar_works/acc/cont/')
         req.environ['wsgi.input'] = open(os.path.join(self.testdir,
                                                       'tar_fails.tar'))
+        req.headers['transfer-encoding'] = 'chunked'
         resp = self.bulk.handle_extract(req, 'gz')
         self.assertEquals(resp.status_int, 400)
         self.assertEquals(self.app.calls, 0)
 
-    def test_extract_tar_fail_max_file(self):
+    def test_extract_tar_fail_max_file_name_length(self):
         tar = self.build_tar()
-        was_failed = self.bulk.max_failed_extractions
-        try:
+        with patch.object(self.bulk, 'max_failed_extractions', 1):
             self.app.calls = 0
-            self.bulk.max_failed_extractions = 1
-            req = Request.blank('/tar_works/acc/cont/')
+            req = Request.blank('/tar_works/acc/cont/',
+                                headers={'Accept': 'application/json'})
             req.environ['wsgi.input'] = open(os.path.join(self.testdir,
                                                           'tar_fails.tar'))
+            req.headers['transfer-encoding'] = 'chunked'
             resp = self.bulk.handle_extract(req, '')
             resp_data = json.loads(resp.body)
             self.assertEquals(self.app.calls, 5)
             self.assertEquals(resp_data['Errors'][0][0],
                               '/tar_works/acc/cont/base_fails1/' + ('f' * 101))
-        finally:
-            self.bulk.max_failed_extractions = was_failed
+
+    @patch.object(bulk, 'MAX_FILE_SIZE', 4)
+    def test_extract_tar_fail_max_file_size(self):
+        tar = self.build_tar()
+        dir_tree = [{'test': [{'sub_dir1': ['sub1_file1']}]}]
+        build_dir_tree(self.testdir, dir_tree)
+        tar = tarfile.open(name=os.path.join(self.testdir,
+                                             'tar_works.tar'),
+                           mode='w')
+        tar.add(os.path.join(self.testdir, 'test'))
+        tar.close()
+        self.app.calls = 0
+        req = Request.blank('/tar_works/acc/cont/',
+                            headers={'Accept': 'application/json'})
+        req.environ['wsgi.input'] = open(
+            os.path.join(self.testdir, 'tar_works.tar'))
+        req.headers['transfer-encoding'] = 'chunked'
+        resp = self.bulk.handle_extract(req, '')
+        resp_data = json.loads(resp.body)
+        self.assert_(resp_data['Errors'][0][1].startswith('413'))
 
     def test_extract_tar_fail_max_cont(self):
         dir_tree = [{'sub_dir1': ['sub1_file1']},
@@ -292,18 +346,14 @@ class TestUntar(unittest.TestCase):
                     'f' * 101,
                     {'sub_dir3': [{'sub4_dir1': 'sub4_file1'}]}]
         tar = self.build_tar(dir_tree)
-        was_max_containers = self.bulk.max_containers
-        try:
+        with patch.object(self.bulk, 'max_containers', 1):
             self.app.calls = 0
-            self.bulk.max_containers = 1
-            req = Request.blank('/tar_works/acc/')
-            req.environ['wsgi.input'] = open(os.path.join(self.testdir,
-                                                          'tar_fails.tar'))
+            body = open(os.path.join(self.testdir, 'tar_fails.tar')).read()
+            req = Request.blank('/tar_works/acc/', body=body)
+            req.headers['transfer-encoding'] = 'chunked'
             resp = self.bulk.handle_extract(req, '')
             self.assertEquals(self.app.calls, 3)
             self.assertEquals(resp.status_int, 400)
-        finally:
-            self.bulk.max_containers = was_max_containers
 
     def test_extract_tar_fail_create_cont(self):
         dir_tree = [{'base_fails1': [
@@ -312,9 +362,11 @@ class TestUntar(unittest.TestCase):
             'f\xde',
             {'./sub_dir3': [{'sub4_dir1': 'sub4_file1'}]}]}]
         tar = self.build_tar(dir_tree)
-        req = Request.blank('/create_cont_fail/acc/cont/')
+        req = Request.blank('/create_cont_fail/acc/cont/',
+                            headers={'Accept': 'application/json'})
         req.environ['wsgi.input'] = open(os.path.join(self.testdir,
                                                       'tar_fails.tar'))
+        req.headers['transfer-encoding'] = 'chunked'
         resp = self.bulk.handle_extract(req, '')
         resp_data = json.loads(resp.body)
         self.assertEquals(self.app.calls, 4)
@@ -322,22 +374,27 @@ class TestUntar(unittest.TestCase):
 
     def test_extract_tar_fail_create_cont_value_err(self):
         tar = self.build_tar()
-        req = Request.blank('/create_cont_fail/acc/cont/')
+        req = Request.blank('/create_cont_fail/acc/cont/',
+                            headers={'Accept': 'application/json'})
         req.environ['wsgi.input'] = open(os.path.join(self.testdir,
                                                       'tar_fails.tar'))
+        req.headers['transfer-encoding'] = 'chunked'
 
         def bad_create(req, path):
             raise ValueError('Test')
 
-        was_func = self.bulk.create_container
-        try:
-            self.bulk.create_container = bad_create
+        with patch.object(self.bulk, 'create_container', bad_create):
             resp = self.bulk.handle_extract(req, '')
             resp_data = json.loads(resp.body)
             self.assertEquals(self.app.calls, 0)
             self.assertEquals(len(resp_data['Errors']), 5)
-        finally:
-            self.bulk.create_container = was_func
+
+    def test_get_response_body(self):
+        self.assertRaises(
+            HTTPException, bulk.get_response_body, 'badformat', {}, [])
+        xml_body = bulk.get_response_body(
+            'text/xml', {'hey': 'there'}, [['json > xml', '202 Accepted']])
+        self.assert_('&gt' in xml_body)
 
 
 class TestDelete(unittest.TestCase):
@@ -351,9 +408,9 @@ class TestDelete(unittest.TestCase):
         self.app.delete_paths = []
 
     def test_bulk_delete_works(self):
-        req = Request.blank('/delete_works/AUTH_Acc')
+        req = Request.blank('/delete_works/AUTH_Acc', body='/c/f\n/c/f404',
+                            headers={'Accept': 'application/json'})
         req.method = 'DELETE'
-        req.environ['wsgi.input'] = StringIO('/c/f\n/c/f404')
         resp = self.bulk.handle_delete(req)
         self.assertEquals(
             self.app.delete_paths,
@@ -363,12 +420,29 @@ class TestDelete(unittest.TestCase):
         self.assertEquals(resp_data['Number Deleted'], 1)
         self.assertEquals(resp_data['Number Not Found'], 1)
 
+    def test_bulk_delete_bad_accept_and_content_type(self):
+        req = Request.blank('/delete_works/AUTH_Acc',
+                            headers={'Accept': 'badformat'})
+
+        req.method = 'DELETE'
+        req.environ['wsgi.input'] = StringIO('/c/f\n/c/f404')
+        resp = self.bulk.handle_delete(req)
+        self.assertEquals(resp.status_int, 406)
+
+        req = Request.blank('/delete_works/AUTH_Acc',
+                            headers={'Accept': 'application/json',
+                                     'Content-Type': 'text/xml'})
+        req.method = 'DELETE'
+        req.environ['wsgi.input'] = StringIO('/c/f\n/c/f404')
+        resp = self.bulk.handle_delete(req)
+        self.assertEquals(resp.status_int, 406)
+
     def test_bulk_delete_call(self):
         def fake_start_response(*args, **kwargs):
             pass
-        req = Request.blank('/delete_works/AUTH_Acc')
+        req = Request.blank('/delete_works/AUTH_Acc?bulk-delete')
         req.method = 'DELETE'
-        req.headers['x-bulk-delete'] = 't'
+        req.headers['Transfer-Encoding'] = 'chunked'
         req.environ['wsgi.input'] = StringIO('/c/f')
         self.bulk(req.environ, fake_start_response)
         self.assertEquals(
@@ -376,45 +450,47 @@ class TestDelete(unittest.TestCase):
         self.assertEquals(self.app.calls, 1)
 
     def test_bulk_delete_get_objs(self):
-        was_max_dels = self.bulk.max_deletes_per_request
-        was_max_path = bulk.MAX_PATH_LENGTH
-        req = Request.blank('/delete_works/AUTH_Acc')
+        req = Request.blank('/delete_works/AUTH_Acc', body='1\r\n2\r\n')
         req.method = 'DELETE'
-        try:
-            self.bulk.max_deletes_per_request = 2
-            req.environ['wsgi.input'] = StringIO('1\r\n2\r\n')
+        with patch.object(self.bulk, 'max_deletes_per_request', 2):
             results = self.bulk.get_objs_to_delete(req)
             self.assertEquals(results, ['1\r', '2\r'])
 
+        with patch.object(bulk, 'MAX_PATH_LENGTH', 2):
             results = []
-            bulk.MAX_PATH_LENGTH = 2
-            self.bulk.max_deletes_per_request = 2
             req.environ['wsgi.input'] = StringIO('1\n2\n3')
             results = self.bulk.get_objs_to_delete(req)
             self.assertEquals(results, ['1', '2', '3'])
 
-        finally:
-            bulk.MAX_PATH_LENGTH = was_max_path
-            self.bulk.max_deletes_per_request = was_max_dels
+        with patch.object(self.bulk, 'max_deletes_per_request', 9):
+            with patch.object(bulk, 'MAX_PATH_LENGTH', 1):
+                req_body = '\n'.join([str(i) for i in xrange(10)])
+                req = Request.blank('/delete_works/AUTH_Acc', body=req_body)
+                self.assertRaises(
+                    HTTPException, self.bulk.get_objs_to_delete, req)
 
-    def test_bulk_delete_works_extra_newlines(self):
-        req = Request.blank('/delete_works/AUTH_Acc')
+    def test_bulk_delete_works_extra_newlines_extra_quoting(self):
+        req = Request.blank('/delete_works/AUTH_Acc',
+                            body='/c/f\n\n\n/c/f404\n\n\n/c/%2525',
+                            headers={'Accept': 'application/json'})
         req.method = 'DELETE'
-        req.environ['wsgi.input'] = StringIO('/c/f\n\n\n/c/f404\n\n\n')
         resp = self.bulk.handle_delete(req)
         self.assertEquals(
             self.app.delete_paths,
-            ['/delete_works/AUTH_Acc/c/f', '/delete_works/AUTH_Acc/c/f404'])
-        self.assertEquals(self.app.calls, 2)
+            ['/delete_works/AUTH_Acc/c/f',
+             '/delete_works/AUTH_Acc/c/f404',
+             '/delete_works/AUTH_Acc/c/%25'])
+        self.assertEquals(self.app.calls, 3)
         resp_data = json.loads(resp.body)
-        self.assertEquals(resp_data['Number Deleted'], 1)
+        self.assertEquals(resp_data['Number Deleted'], 2)
         self.assertEquals(resp_data['Number Not Found'], 1)
 
     def test_bulk_delete_too_many_newlines(self):
         req = Request.blank('/delete_works/AUTH_Acc')
         req.method = 'DELETE'
-        req.environ['wsgi.input'] = \
-            StringIO('\n\n' * self.bulk.max_deletes_per_request)
+        data = '\n\n' * self.bulk.max_deletes_per_request
+        req.environ['wsgi.input'] = StringIO(data)
+        req.content_length = len(data)
         try:
             self.bulk.handle_delete(req)
         except HTTPException, err:
@@ -426,20 +502,20 @@ class TestDelete(unittest.TestCase):
         def fake_start_response(*args, **kwargs):
             self.assertTrue(args[0].startswith('413'))
 
-        req = Request.blank('/delete_works/AUTH_Acc')
+        req = Request.blank('/delete_works/AUTH_Acc?bulk-delete')
         req.method = 'DELETE'
-        req.headers['x-bulk-delete'] = 't'
-        req.environ['wsgi.input'] = \
-            StringIO('\n\n' * self.bulk.max_deletes_per_request)
+        data = '\n\n' * self.bulk.max_deletes_per_request
+        req.environ['wsgi.input'] = StringIO(data)
+        req.content_length = len(data)
         self.bulk(req.environ, fake_start_response)
 
     def test_bulk_delete_works_unicode(self):
-        req = Request.blank('/delete_works/AUTH_Acc')
+        body = (u'/c/ obj \u2661\r\n'.encode('utf8') +
+                'c/ objbadutf8\r\n' +
+                '/c/f\xdebadutf8\n')
+        req = Request.blank('/delete_works/AUTH_Acc', body=body,
+                            headers={'Accept': 'application/json'})
         req.method = 'DELETE'
-        req.environ['wsgi.input'] = \
-            StringIO(u'/c/ obj \u2661\r\n'.encode('utf8') +
-                     'c/ objbadutf8\r\n' +
-                     '/c/f\xdebadutf8\n')
         resp = self.bulk.handle_delete(req)
         self.assertEquals(
             self.app.delete_paths,
@@ -452,27 +528,29 @@ class TestDelete(unittest.TestCase):
         self.assertEquals(len(resp_data['Errors']), 2)
         self.assertEquals(
             resp_data['Errors'],
-            [['/delete_works/AUTH_Acc/c/ objbadutf8',
+            [[urllib.quote('/delete_works/AUTH_Acc/c/ objbadutf8'),
               '412 Precondition Failed'],
              [urllib.quote('/delete_works/AUTH_Acc/c/f\xdebadutf8'),
               '412 Precondition Failed']])
 
     def test_bulk_delete_no_body(self):
         req = Request.blank('/unauth/AUTH_acc/')
+        self.assertRaises(HTTPException, self.bulk.handle_delete, req)
+
+    def test_bulk_delete_no_files_in_body(self):
+        req = Request.blank('/unauth/AUTH_acc/', body=' ')
         resp = self.bulk.handle_delete(req)
         self.assertEquals(resp.status_int, 400)
 
     def test_bulk_delete_unauth(self):
-        req = Request.blank('/unauth/AUTH_acc/')
+        req = Request.blank('/unauth/AUTH_acc/', body='/c/f\n')
         req.method = 'DELETE'
-        req.environ['wsgi.input'] = StringIO('/c/f\n')
         resp = self.bulk.handle_delete(req)
         self.assertEquals(resp.status_int, 401)
 
     def test_bulk_delete_500_resp(self):
-        req = Request.blank('/broke/AUTH_acc/')
+        req = Request.blank('/broke/AUTH_acc/', body='/c/f\n')
         req.method = 'DELETE'
-        req.environ['wsgi.input'] = StringIO('/c/f\n')
         resp = self.bulk.handle_delete(req)
         self.assertEquals(resp.status_int, 502)
 
@@ -482,29 +560,30 @@ class TestDelete(unittest.TestCase):
         self.assertEquals(resp.status_int, 404)
 
     def test_bulk_delete_container_delete(self):
-        req = Request.blank('/delete_cont_fail/AUTH_Acc')
+        req = Request.blank('/delete_cont_fail/AUTH_Acc', body='c\n',
+                            headers={'Accept': 'application/json'})
         req.method = 'DELETE'
-        req.environ['wsgi.input'] = StringIO('c\n')
         resp = self.bulk.handle_delete(req)
         resp_data = json.loads(resp.body)
         self.assertEquals(resp_data['Number Deleted'], 0)
         self.assertEquals(resp_data['Errors'][0][1], '409 Conflict')
 
     def test_bulk_delete_bad_file_too_long(self):
-        req = Request.blank('/delete_works/AUTH_Acc')
+        req = Request.blank('/delete_works/AUTH_Acc',
+                            headers={'Accept': 'application/json'})
         req.method = 'DELETE'
-        req.environ['wsgi.input'] = \
-            StringIO('/c/f\nc/' + ('1' * bulk.MAX_PATH_LENGTH) + '\n/c/f')
+        data = '/c/f\nc/' + ('1' * bulk.MAX_PATH_LENGTH) + '\n/c/f'
+        req.environ['wsgi.input'] = StringIO(data)
+        req.headers['Transfer-Encoding'] = 'chunked'
         resp = self.bulk.handle_delete(req)
         resp_data = json.loads(resp.body)
         self.assertEquals(resp_data['Number Deleted'], 2)
         self.assertEquals(resp_data['Errors'][0][1], '400 Bad Request')
 
     def test_bulk_delete_bad_file_over_twice_max_length(self):
-        req = Request.blank('/delete_works/AUTH_Acc')
+        body = '/c/f\nc/' + ('123456' * bulk.MAX_PATH_LENGTH) + '\n'
+        req = Request.blank('/delete_works/AUTH_Acc', body=body)
         req.method = 'DELETE'
-        req.environ['wsgi.input'] = \
-            StringIO('/c/f\nc/' + ('123456' * bulk.MAX_PATH_LENGTH) + '\n')
         try:
             self.bulk.handle_delete(req)
         except HTTPException, err:
