@@ -14,6 +14,7 @@
 # limitations under the License.
 
 import unittest
+import time
 from urllib import quote, unquote
 import cStringIO as StringIO
 from logging.handlers import SysLogHandler
@@ -25,12 +26,14 @@ from swift.common.swob import Request
 
 
 class FakeApp(object):
-    def __init__(self, body=['FAKE APP']):
+    def __init__(self, body=['FAKE APP'], response_str='200 OK'):
         self.body = body
+        self.response_str = response_str
 
     def __call__(self, env, start_response):
-        start_response('200 OK', [('Content-Type', 'text/plain'),
-                            ('Content-Length', str(sum(map(len, self.body))))])
+        start_response(self.response_str,
+                       [('Content-Type', 'text/plain'),
+                        ('Content-Length', str(sum(map(len, self.body))))])
         while env['wsgi.input'].read(5):
             pass
         return self.body
@@ -72,19 +75,48 @@ def start_response(*args):
 
 class TestProxyLogging(unittest.TestCase):
 
-    def _log_parts(self, app):
+    def _log_parts(self, app, should_be_empty=False):
         info_calls = app.access_logger.log_dict['info']
-        self.assertEquals(1, len(info_calls))
-        return info_calls[0][0][0].split(' ')
+        if should_be_empty:
+            self.assertEquals([], info_calls)
+        else:
+            self.assertEquals(1, len(info_calls))
+            return info_calls[0][0][0].split(' ')
 
     def assertTiming(self, exp_metric, app, exp_timing=None):
         timing_calls = app.access_logger.log_dict['timing']
-        self.assertEquals(1, len(timing_calls))
-        self.assertEquals({}, timing_calls[0][1])
-        self.assertEquals(2, len(timing_calls[0][0]))
-        self.assertEquals(exp_metric, timing_calls[0][0][0])
-        if exp_timing is not None:
-            self.assertEquals(exp_timing, timing_calls[0][0][1])
+        found = False
+        for timing_call in timing_calls:
+            self.assertEquals({}, timing_call[1])
+            self.assertEquals(2, len(timing_call[0]))
+            if timing_call[0][0] == exp_metric:
+                found = True
+                if exp_timing is not None:
+                    self.assertAlmostEqual(exp_timing, timing_call[0][1],
+                                           places=4)
+        if not found:
+            self.assertTrue(False, 'assertTiming: %s not found in %r' % (
+                exp_metric, timing_calls))
+
+    def assertTimingSince(self, exp_metric, app, exp_start=None):
+        timing_calls = app.access_logger.log_dict['timing_since']
+        found = False
+        for timing_call in timing_calls:
+            self.assertEquals({}, timing_call[1])
+            self.assertEquals(2, len(timing_call[0]))
+            if timing_call[0][0] == exp_metric:
+                found = True
+                if exp_start is not None:
+                    self.assertAlmostEqual(exp_start, timing_call[0][1],
+                                           places=4)
+        if not found:
+            self.assertTrue(False, 'assertTimingSince: %s not found in %r' % (
+                exp_metric, timing_calls))
+
+    def assertNotTiming(self, not_exp_metric, app):
+        timing_calls = app.access_logger.log_dict['timing']
+        for timing_call in timing_calls:
+            self.assertNotEqual(not_exp_metric, timing_call[0][0])
 
     def assertUpdateStats(self, exp_metric, exp_bytes, app):
         update_stats_calls = app.access_logger.log_dict['update_stats']
@@ -99,9 +131,8 @@ class TestProxyLogging(unittest.TestCase):
             req = Request.blank(url, environ={'REQUEST_METHOD': 'GET'})
             resp = app(req.environ, start_response)
             resp_body = ''.join(resp)
-            self.assertEquals(0, len(app.access_logger.log_dict['timing']))
-            self.assertEquals(0,
-                              len(app.access_logger.log_dict['update_stats']))
+            self.assertEqual([], app.access_logger.log_dict['timing'])
+            self.assertEqual([], app.access_logger.log_dict['update_stats'])
 
     def test_log_request_stat_type_bad(self):
         for bad_path in ['', '/', '/bad', '/baddy/mc_badderson', '/v1',
@@ -109,11 +140,21 @@ class TestProxyLogging(unittest.TestCase):
             app = proxy_logging.ProxyLoggingMiddleware(FakeApp(), {})
             app.access_logger = FakeLogger()
             req = Request.blank(bad_path, environ={'REQUEST_METHOD': 'GET'})
-            app.log_request(req.environ, 123, 7, 13, 2.71828182846, False)
+            app.log_request(req, 123, 7, 13, 2.71828182846)
             self.assertEqual([], app.access_logger.log_dict['timing'])
             self.assertEqual([], app.access_logger.log_dict['update_stats'])
 
     def test_log_request_stat_type_good(self):
+        """
+        log_request() should send timing and byte-count counters for GET
+        requests.  Also, __call__()'s iter_response() function should
+        statsd-log time to first byte (calling the passed-in start_response
+        function), but only for GET requests.
+        """
+        stub_times = []
+        def stub_time():
+            return stub_times.pop(0)
+
         path_types = {
             '/v1/a': 'account',
             '/v1/a/': 'account',
@@ -126,13 +167,62 @@ class TestProxyLogging(unittest.TestCase):
             '/v1/a/c/o/p/p2': 'object',
         }
         for path, exp_type in path_types.iteritems():
-            app = proxy_logging.ProxyLoggingMiddleware(FakeApp(), {})
-            app.access_logger = FakeLogger()
-            req = Request.blank(path, environ={'REQUEST_METHOD': 'GET'})
-            app.log_request(req.environ, 321, 7, 13, 2.71828182846, False)
-            self.assertTiming('%s.GET.321.timing' % exp_type, app,
-                              exp_timing=2.71828182846 * 1000)
-            self.assertUpdateStats('%s.GET.321.xfer' % exp_type, 7 + 13, app)
+            orig_time = time.time
+            try:
+                time.time = stub_time
+                # GET
+                app = proxy_logging.ProxyLoggingMiddleware(
+                    FakeApp(body='7654321', response_str='321 Fubar'), {})
+                app.access_logger = FakeLogger()
+                req = Request.blank(path, environ={
+                    'REQUEST_METHOD': 'GET',
+                    'wsgi.input': StringIO.StringIO('4321')})
+                stub_times = [18.0, 20.71828182846]
+                iter_response = app(req.environ, lambda *_: None)
+                self.assertEqual('7654321', ''.join(iter_response))
+                self.assertTiming('%s.GET.321.timing' % exp_type, app,
+                                  exp_timing=2.71828182846 * 1000)
+                self.assertTimingSince(
+                    '%s.GET.321.first-byte.timing' % exp_type, app,
+                    exp_start=18.0)
+                self.assertUpdateStats('%s.GET.321.xfer' % exp_type,
+                                       4 + 7, app)
+
+                # GET with swift.proxy_access_log_made already set
+                app = proxy_logging.ProxyLoggingMiddleware(
+                    FakeApp(body='7654321', response_str='321 Fubar'), {})
+                app.access_logger = FakeLogger()
+                req = Request.blank(path, environ={
+                    'REQUEST_METHOD': 'GET',
+                    'swift.proxy_access_log_made': True,
+                    'wsgi.input': StringIO.StringIO('4321')})
+                stub_times = [18.0, 20.71828182846]
+                iter_response = app(req.environ, lambda *_: None)
+                self.assertEqual('7654321', ''.join(iter_response))
+                self.assertEqual([], app.access_logger.log_dict['timing'])
+                self.assertEqual([], app.access_logger.log_dict['timing_since'])
+                self.assertEqual([], app.access_logger.log_dict['update_stats'])
+
+                # PUT (no first-byte timing!)
+                app = proxy_logging.ProxyLoggingMiddleware(
+                    FakeApp(body='87654321', response_str='314 PiTown'), {})
+                app.access_logger = FakeLogger()
+                req = Request.blank(path, environ={
+                    'REQUEST_METHOD': 'PUT',
+                    'wsgi.input': StringIO.StringIO('654321')})
+                # (it's not a GET, so time() doesn't have a 2nd call)
+                stub_times = [58.2, 58.2 + 7.3321]
+                iter_response = app(req.environ, lambda *_: None)
+                self.assertEqual('87654321', ''.join(iter_response))
+                self.assertTiming('%s.PUT.314.timing' % exp_type, app,
+                                  exp_timing=7.3321 * 1000)
+                self.assertNotTiming(
+                    '%s.GET.314.first-byte.timing' % exp_type, app)
+                self.assertNotTiming(
+                    '%s.PUT.314.first-byte.timing' % exp_type, app)
+                self.assertUpdateStats('%s.PUT.314.xfer' % exp_type, 6 + 8, app)
+            finally:
+                time.time = orig_time
 
     def test_log_request_stat_method_filtering_default(self):
         method_map = {
@@ -152,7 +242,7 @@ class TestProxyLogging(unittest.TestCase):
             app = proxy_logging.ProxyLoggingMiddleware(FakeApp(), {})
             app.access_logger = FakeLogger()
             req = Request.blank('/v1/a/', environ={'REQUEST_METHOD': method})
-            app.log_request(req.environ, 299, 11, 3, 1.17, False)
+            app.log_request(req, 299, 11, 3, 1.17)
             self.assertTiming('account.%s.299.timing' % exp_method, app,
                               exp_timing=1.17 * 1000)
             self.assertUpdateStats('account.%s.299.xfer' % exp_method,
@@ -177,7 +267,7 @@ class TestProxyLogging(unittest.TestCase):
                 })
                 app.access_logger = FakeLogger()
                 req = Request.blank('/v1/a/c', environ={'REQUEST_METHOD': method})
-                app.log_request(req.environ, 911, 4, 43, 1.01, False)
+                app.log_request(req, 911, 4, 43, 1.01)
                 self.assertTiming('container.%s.911.timing' % exp_method, app,
                                 exp_timing=1.01 * 1000)
                 self.assertUpdateStats('container.%s.911.xfer' % exp_method,
@@ -197,6 +287,17 @@ class TestProxyLogging(unittest.TestCase):
         self.assertEquals(resp_body, 'FAKE APP')
         self.assertEquals(log_parts[11], str(len(resp_body)))
 
+    def test_basic_req_second_time(self):
+        app = proxy_logging.ProxyLoggingMiddleware(FakeApp(), {})
+        app.access_logger = FakeLogger()
+        req = Request.blank('/', environ={
+            'swift.proxy_access_log_made': True,
+            'REQUEST_METHOD': 'GET'})
+        resp = app(req.environ, start_response)
+        resp_body = ''.join(resp)
+        log_parts = self._log_parts(app, should_be_empty=True)
+        self.assertEquals(resp_body, 'FAKE APP')
+
     def test_multi_segment_resp(self):
         app = proxy_logging.ProxyLoggingMiddleware(FakeApp(
             ['some', 'chunks', 'of data']), {})
@@ -334,7 +435,7 @@ class TestProxyLogging(unittest.TestCase):
         try:
             resp = app(req.environ, start_response)
             body = ''.join(resp)
-        except Exception:
+        except IOError:
             pass
         log_parts = self._log_parts(app)
         self.assertEquals(log_parts[6], '499')
@@ -349,7 +450,7 @@ class TestProxyLogging(unittest.TestCase):
         try:
             resp = app(req.environ, start_response)
             body = ''.join(resp)
-        except Exception:
+        except IOError:
             pass
         log_parts = self._log_parts(app)
         self.assertEquals(log_parts[6], '499')
