@@ -23,6 +23,7 @@ from swiftclient import ClientException
 
 
 utils.HASH_PATH_SUFFIX = 'endcap'
+utils.HASH_PATH_PREFIX = 'endcap'
 
 
 class FakeRing(object):
@@ -154,7 +155,7 @@ class TestContainerSync(unittest.TestCase):
 
     def test_run_once(self):
         # This runs runs_once with fakes twice, the first causing an interim
-        # report, the second with no interm report.
+        # report, the second with no interim report.
         time_calls = [0]
         audit_location_generator_calls = [0]
 
@@ -422,10 +423,10 @@ class TestContainerSync(unittest.TestCase):
             cs.allowed_sync_hosts = ['127.0.0.1']
             cs.container_sync('isa.db')
             # Succeeds because no rows match
-            self.assertEquals(cs.container_failures, 0)
+            self.assertEquals(cs.container_failures, 1)
             self.assertEquals(cs.container_skips, 0)
             self.assertEquals(fcb.sync_point1, None)
-            self.assertEquals(fcb.sync_point2, 1)
+            self.assertEquals(fcb.sync_point2, -1)
 
             def fake_hash_path(account, container, obj, raw_digest=False):
                 # Ensures that all rows match for full syncing, ordinal is 0
@@ -446,7 +447,7 @@ class TestContainerSync(unittest.TestCase):
             cs.allowed_sync_hosts = ['127.0.0.1']
             cs.container_sync('isa.db')
             # Succeeds because the two sync points haven't deviated yet
-            self.assertEquals(cs.container_failures, 0)
+            self.assertEquals(cs.container_failures, 1)
             self.assertEquals(cs.container_skips, 0)
             self.assertEquals(fcb.sync_point1, -1)
             self.assertEquals(fcb.sync_point2, -1)
@@ -465,9 +466,9 @@ class TestContainerSync(unittest.TestCase):
             cs.container_sync('isa.db')
             # Fails because container_sync_row will fail since the row has no
             # 'deleted' key
-            self.assertEquals(cs.container_failures, 1)
+            self.assertEquals(cs.container_failures, 2)
             self.assertEquals(cs.container_skips, 0)
-            self.assertEquals(fcb.sync_point1, -1)
+            self.assertEquals(fcb.sync_point1, None)
             self.assertEquals(fcb.sync_point2, -1)
 
             fcb = FakeContainerBroker('path',
@@ -484,9 +485,9 @@ class TestContainerSync(unittest.TestCase):
             cs.allowed_sync_hosts = ['127.0.0.1']
             cs.container_sync('isa.db')
             # Fails because delete_object fails
-            self.assertEquals(cs.container_failures, 2)
+            self.assertEquals(cs.container_failures, 3)
             self.assertEquals(cs.container_skips, 0)
-            self.assertEquals(fcb.sync_point1, -1)
+            self.assertEquals(fcb.sync_point1, None)
             self.assertEquals(fcb.sync_point2, -1)
 
             def fake_delete_object(*args, **kwargs):
@@ -507,7 +508,7 @@ class TestContainerSync(unittest.TestCase):
             cs.allowed_sync_hosts = ['127.0.0.1']
             cs.container_sync('isa.db')
             # Succeeds because delete_object succeeds
-            self.assertEquals(cs.container_failures, 2)
+            self.assertEquals(cs.container_failures, 3)
             self.assertEquals(cs.container_skips, 0)
             self.assertEquals(fcb.sync_point1, None)
             self.assertEquals(fcb.sync_point2, 1)
@@ -574,10 +575,11 @@ class TestContainerSync(unittest.TestCase):
             cs.allowed_sync_hosts = ['127.0.0.1']
             cs.container_sync('isa.db')
             # Fails because row is missing 'deleted' key
+            # Nevertheless the fault is skipped
             self.assertEquals(cs.container_failures, 1)
             self.assertEquals(cs.container_skips, 0)
-            self.assertEquals(fcb.sync_point1, -1)
-            self.assertEquals(fcb.sync_point2, -1)
+            self.assertEquals(fcb.sync_point1, 1)
+            self.assertEquals(fcb.sync_point2, None)
 
             fcb = FakeContainerBroker('path',
                 info={'account': 'a', 'container': 'c',
